local STI = require("lib/sti")
require("player")
require("window")
require("playercamera")

-- the round function from lua programming book
local floor = math.floor
function Round(x)
    local f = floor(x + 0.5)
    if (x == f) or (x % 2.0 == 0.5) then
        return f
    else
        return floor(x + 0.5)
    end
end

function love.load()
    -- Window
    Window:load(1) -- Pass in the scale, multiple of 480 x 270
    -- Map
    Map = STI("assets/maps/main.lua", { "box2d" })
    World = love.physics.newWorld(0, 0)
    World:setCallbacks(beginContact, endContact)
    Map:box2d_init(World)
    -- Player
    Player:load()
    PlayerCamera:load()
end

function love.update(dt)
    World:update(dt)
    Player:update(dt)
    PlayerCamera:update(dt)
    love.keyboard.resetKeyStates()
end

function love.draw()
    -- Test
    local width = 4
    local height = 8

    PlayerCamera:attach()
    Window:draw()
    Map:drawLayer(Map.layers["ground"])
    Map:drawLayer(Map.layers["overworld"])
    -- Map:drawLayer(Map.layers["solid"])
    Player:draw()
    -- Test
    -- Top Left World

    love.graphics.rectangle(
        "fill",
        0, -- X world coordinates
        0, -- Y world coordinates
        width,
        height
    )
    -- Bottom left
    mapH = Map.height * Map.tileheight
    love.graphics.rectangle(
        "fill",
        0,
        mapH - 20,
        width,
        20
    )
    -- Top right of the world
    mapW = Map.width * Map.tilewidth
    love.graphics.rectangle(
        "fill",
        mapW - width,
        0,
        width,
        20
    )
    -- Bottom right of the world
    love.graphics.rectangle(
        "fill",
        mapW - width,
        mapH - height,
        width,
        height
    )
    PlayerCamera:detach()
    -- Test draw on camera
    local windowW = love.graphics.getWidth()
    local windowH = love.graphics.getHeight()
    -- Top left camera
    love.graphics.rectangle(
        "fill",
        0, -- X screen coordinates
        0, -- Y screen coordinates
        width,
        height
    )
    -- Bottom left camera
    love.graphics.rectangle(
        "fill",
        0,
        windowH - height,
        width,
        height
    )
    -- Top right camera
    love.graphics.rectangle(
        "fill",
        windowW - width,
        0,
        width,
        height
    )
    -- Bottom right camera
    love.graphics.rectangle(
        "fill",
        windowW - width,
        windowH - height,
        width,
        height
    )
end
