require("input")
require("utils")
anim8 = require("lib/anim8/anim8")

Player = {}

function Player:load()
    self.canMove = true
    self.x = 30 * 16 / 2 -- Number of tiles * tile width / 2 for half TODO: Add this to window or map.lua?
    self.y = 17 * 16 / 2 -- Number of tiles * tile width / 2 for half TODO: Add this to window or map.lua?
    self.width = 16
    self.height = 22
    self.vx = 0
    self.vy = 0
    self.speed = 0
    self.max_speed = 150
    self.acceleration = 2000
    self.deceleration = 2000
    self.isMoving = false
    -- Physics
    self.physics = {}
    self.physics.body = love.physics.newBody(World, self.x, self.y, "dynamic")
    self.physics.body:setFixedRotation(true)
    self.physics.shape = love.physics.newRectangleShape(self.width, self.height)
    self.physics.fixture = love.physics.newFixture(self.physics.body, self.physics.shape)
    -- Anim
    self.spriteSheet = love.graphics.newImage("assets/images/character.png")
    self.grid = anim8.newGrid(16, 32, self.spriteSheet:getWidth(), self.spriteSheet:getHeight())
    self.animations = {}
    self.animations.down = anim8.newAnimation(self.grid('1-4', 1), 0.2)
    self.animations.right = anim8.newAnimation(self.grid('1-4', 2), 0.2)
    self.animations.up = anim8.newAnimation(self.grid('1-4', 3), 0.2)
    self.animations.left = anim8.newAnimation(self.grid('1-4', 4), 0.2)
    self.anim = self.animations.left
end

function Player:update(dt, ignorePlayerInput)
    local dir = getDirection()
    self:move(dt, dir)
    self:syncPhysics()
    self.anim:update(dt)
    self:idle()
end

function Player:move_horizontal(dt, dir)
    -- Move left
    if dir.x == -1 then
        self.vx = math.max(self.vx - self.acceleration * dt, -self.max_speed)
        self.anim = self.animations.left
        -- Move right
    elseif dir.x == 1 then
        self.vx = math.min(self.vx + self.acceleration * dt, self.max_speed)
        self.anim = self.animations.right
    else
        -- Apply deceleration right
        if self.vx > 0 then
            self.vx = math.max(self.vx - self.deceleration * dt, 0)
            -- Apply deceleration left
        elseif self.vx < 0 then
            self.vx = math.min(self.vx + self.deceleration * dt, 0)
        end
    end
end

function Player:move_vertical(dt, dir)
    if dir.y == -1 then
        self.vy = math.max(self.vy - self.acceleration * dt, -self.max_speed)
        self.anim = self.animations.up
    elseif dir.y == 1 then
        self.vy = math.min(self.vy + self.acceleration * dt, self.max_speed)
        self.anim = self.animations.down
    else
        if self.vy > 0 then
            self.vy = math.max(self.vy - self.deceleration * dt, 0)
        elseif self.vy < 0 then
            self.vy = math.min(self.vy + self.deceleration * dt, 0)
        end
    end
end

function Player:move(dt, dir)
    if dir.x == 0 and dir.y == 0 then
        self.isMoving = false
    else
        self.isMoving = true
    end
    self:move_horizontal(dt, dir)
    self:move_vertical(dt, dir)
    -- Normalize the velocity vector to prevent player from moving fast on the diagonal
    local magnitude = math.sqrt(self.vx ^ 2 + self.vy ^ 2)
    if magnitude > self.max_speed then
        local scale = self.max_speed / magnitude
        self.vx = self.vx * scale
        self.vy = self.vy * scale
    end
end

function Player:idle()
    if not self.isMoving then
        self.anim:gotoFrame(1)
    end
end

function Player:syncPhysics()
    self.x, self.y = self.physics.body:getPosition()
    self.physics.body:setLinearVelocity(self.vx, self.vy)
end

function Player:draw()
    -- love.graphics.rectangle("line", self.x - self.width / 2, self.y - self.height / 2, self.width, self.height)
    -- love.graphics.setColor(255, 255, 255, 255)
    self.anim:draw(self.spriteSheet, Round(self.x), Round(self.y), nil, nil, nil, 8, 16)
end
